from enum import Enum


def version():
    return "0.2.0"


class Language(Enum):
    ENGLISH = 'English'
    RUSSIAN = 'Russian'


__current_language = Language.RUSSIAN


def set_current_language(value: str):
    global __current_language
    __current_language = Language(value)


def get_current_language() -> Language:
    return __current_language


def get_supporting_language() -> list[str]:
    return [language.name for language in Language]



from ...types import common_data_types as cdt


class DoubleLongUnsignedSecond(cdt.DoubleLongUnsigned):
    """for second implementation"""
    SCALER_UNIT = cdt.ScalUnitType((0, 7))


class IPAddress(cdt.DoubleLongUnsigned):
    """with string parser"""

    def from_str(self, value: str) -> bytes:
        """ create ip: integer from string type ddd.ddd.ddd.ddd, ex.: 127.0.0.1 """
        raw_value = bytes()
        for separator in '... ':
            try:
                element, value = value.split(separator, 1)
            except ValueError:
                element, value = value, ''
            raw_value += self.__get_attr_element(element)
        return raw_value

    @staticmethod
    def __get_attr_element(value: str) -> bytes:
        if isinstance(value, str):
            if value == '':
                return b'\x00'
            try:
                return int(value).to_bytes(1, 'big')
            except OverflowError:
                raise ValueError(F'Int too big to convert {value}')
        else:
            raise TypeError(F'Unsupported type validation from string, got {value.__class__}')

    def __str__(self):
        return F'{self.contents[0]}.{self.contents[1]}.{self.contents[2]}.{self.contents[3]}'



import unittest
from src.DLMS_SPODES.hdlc import frame


class TestType(unittest.TestCase):
    def test_Address(self):
        ad1 = frame.Address(
            upper_address=0x1,
            lower_address=None)
        self.assertEqual(ad1.content, b'\x03', "upper only")
        ad2 = frame.Address(content=b'\x07')
        self.assertEqual((ad2.upper, ad2.lower), (3, None), "1length from content")
        ad3 = frame.Address(
            upper_address=0x1,
            lower_address=0,
            length=2
        )
        self.assertEqual(ad3.content, b'\x02\x01', "upper only")
        self.assertEqual((ad3.upper, ad3.lower), (1, 0), "lower set to 0")
        ad4 = frame.Address(
            upper_address=0x1,
            lower_address=0x10,
            length=4
        )
        self.assertEqual(ad4.content, b'\x00\x02\x00\x21', "rise to 4")
        ad5 = frame.Address(
            upper_address=0x100,
            lower_address=0x10,
            length=4
        )
        self.assertEqual(ad5.content, b'\x04\x00\x00\x21', "4 length address")



from struct import pack
from functools import cached_property


class LLC:
    __DESTINATION__LSAP: int = 0xe6
    __BROADCAST: int = 0xff
    __COMMAND: int = 0xe6
    __RESPONSE: int = 0xe7
    __QUALITY: int = 0x00
    __destination_lsap: int
    __source_lsap: int
    __info: bytes

    def __init__(self, content: bytearray = None,
                 message: bytes = None):
        if message is None:
            self.__destination_lsap = content.pop(0)
            if self.__destination_lsap not in (self.__DESTINATION__LSAP, self.__BROADCAST):
                raise ValueError(F'Destination tag wrong, expected {hex(self.__DESTINATION__LSAP), hex(self.__BROADCAST)}, got {hex(self.__destination_lsap)}')
            else:
                self.__source_lsap = content.pop(0)
                if self.__source_lsap not in (self.__COMMAND, self.__RESPONSE):
                    raise ValueError(F'Destination tag wrong, expected {hex(self.__COMMAND)}, {hex(self.__RESPONSE)}, got {hex(self.__source_lsap)}')
                else:
                    quality = content.pop(0)
                    if quality != self.__QUALITY:
                        raise ValueError(F'Quality tag wrong, expected {hex(self.__QUALITY)}, got {hex(quality)}')
                    else:
                        self.__info = bytes(content)
        else:
            self.__destination_lsap = self.__DESTINATION__LSAP
            self.__source_lsap = self.__COMMAND
            self.__info = message

    @property
    def info(self) -> bytes:
        return self.__info

    @cached_property
    def content(self) -> bytes:
        return pack('BBB', self.__destination_lsap, self.__source_lsap, self.__QUALITY) + self.__info

    def __str__(self):
        return F'{"broadcast " if self.__destination_lsap == self.__BROADCAST else ""}' \
               F'{"command" if self.__source_lsap == self.__COMMAND else "response"} {self.__info[:10].hex(" ")}'

    def __len__(self):
        return len(self.content)


if __name__ == '__main__':
    l = LLC(content=bytearray((0xe6, 0xe7, 0, 1, 2, 3)))
    print(l)



import unittest
from itertools import count
from src.DLMS_SPODES.types.common_data_types import encode_length
from src.DLMS_SPODES.cosem_interface_classes import ic, collection
from src.DLMS_SPODES.types import cdt, cst, ut, implementations as impl, choices
from src.DLMS_SPODES import relation_to_OBIS, enums
from src.DLMS_SPODES.cosem_interface_classes.collection import Collection


class TestType(unittest.TestCase):
    def test_show_names(self):
        for s in cdt.Structure.__subclasses__():
            print(s.__name__, s(), cdt.get_type_name(s))
            for el in s.ELEMENTS:
                print(F"{el.NAME}: {el}")

    def test_WeekProfile(self):
        from src.DLMS_SPODES.cosem_interface_classes.activity_calendar import WeekProfile
        value = WeekProfile(("00", 1, 1, 1, 1, 1, 1, 1))
        self.assertEqual(value.decode(), (b'\x00', 1, 1, 1, 1, 1, 1, 1), "check decoding")
        print(value.decode()[1:])
        a = value[0]
        print(a)

    def test_KPZPingTestSetup(self):
        col = collection.Collection()
        obj = col.add(
            class_id=ut.CosemClassId(1),
            version=cdt.Unsigned(0),
            logical_name=cst.LogicalName("0.128.154.0.0.255"))
        obj.set_attr(2, 2)
        obj.set_attr(2, b'\x02\x04\x12\x00\x08\x09\x06\x00\x00\x01\x00\x00\xff\x0f\x02\x12\x00\x00')
        print(obj)



import unittest
from src.DLMS_SPODES.types import cdt, cst, ut
from src.DLMS_SPODES.cosem_interface_classes import collection, overview, ic
from src.DLMS_SPODES.cosem_interface_classes import implementations as impl
from src.DLMS_SPODES.version import AppVersion
from itertools import chain


class TestType(unittest.TestCase):

    def test_get_index_and_attr(self):
        obj = collection.Data("0.0.0.1.0.255")
        obj.set_attr(2, cdt.Unsigned(8).encoding)
        self.assertEqual(2, len(list(obj.get_index_with_attributes())), "return amount")

    def test_nothing(self):
        col = collection.get_collection(
            manufacturer=b"KPZ",
            server_type=cdt.OctetString("4d324d5f33"),
            server_ver=AppVersion.from_str("1.4.15"))
        obj = col.get_object("0.0.96.1.4.255")
        print(obj)

    def test_ICAElement(self):
        el = ic.ICAElement(
            NAME="name",
            DATA_TYPE=cdt.OctetString,
            min=1,
            max=100,
            classifier=ic.Classifier.NOT_SPECIFIC)

        el2 = el.get_change(
            classifier=ic.Classifier.STATIC)
        print(el)
        print(el2)

    def test_Exceptions(self):
        raise ic.ObjectValidationError(ln=cst.LogicalName("1.1.1.1.1.1"), i=2, message="some error")

    def test_am_names(self):
        for c in ic.COSEMInterfaceClasses.__subclasses__():
            print(c)
            try:
                for i in chain(c.A_ELEMENTS, c.M_ELEMENTS):
                    print(F"{i}")
            except AttributeError as e:
                print(F"skip {c}: {e}")



from __future__ import annotations
from .__class_init__ import *
from ..types import choices


class Data(ic.COSEMInterfaceClasses):
    """ Object stores data related to internal meter object(s). The meaning of the value is identified by the logical_name.
    The data type of the value is CHOICE. “Data” is typically used to store configuration data and parameters """
    CLASS_ID = ClassID.DATA
    VERSION = Version.V0
    A_ELEMENTS = ic.ICAElement("value", choices.common_dt, classifier=ic.Classifier.NOT_SPECIFIC),

    def characteristics_init(self):
        """nothing do it"""

    @property
    def value(self) -> cdt.CommonDataTypes:
        """Contains the data"""
        return self.get_attr(2)



from .__class_init__ import *


class TCPUDPSetup(ic.COSEMInterfaceClasses):
    """ This IC allows modelling the setup of the TCP or UDP sub-layer of the COSEM TCP or UDP  based transport layer of a TCP-UDP/IP based communication profile.
     In TCP-UDP/IP based communication profiles, all AAs between a physical device hosting one or more COSEM client application processes and a physical device hosting one or more
     COSEM server APs rely on a single TCP or UDP  connection. The TCP or UDP entity is wrapped in the COSEM TCP-UDP based transport layer. Within a physical device, each
     AP - client AP or server logical device - is bound to a Wrapper Port (WPort). The binding is done with the help of the SAP Assignment object. See 4.4.5 DLMS 1000-1 Ed. 12.0.
     On the other hand, COSEM TCP  or UDP based transport layer may be capable to support more than one TCP or UDP connections, between a physical device and several peer physical
     devices hosting COSEM APs.
     When a COSEM physical device supports various data link layers - for example Ethernet and PPP - an instance of the TCP-UDP setup object is necessary for each of them. """
    CLASS_ID = ClassID.TCP_UDP_SETUP
    VERSION = Version.V0
    A_ELEMENTS = (ic.ICAElement("TCP_UDP_port", cdt.LongUnsigned, default=4059),
                  ic.ICAElement("IP_reference", cst.LogicalName),
                  ic.ICAElement("MMS", cdt.LongUnsigned, 40, 535, 535),  # TODO: max, def not according by BlueBook
                  ic.ICAElement("nb_of_sim_conn", cdt.Unsigned, 1),
                  ic.ICAElement("inactivity_time_out", cdt.LongUnsigned, default=180))

    def characteristics_init(self):
        """nothing do it"""

    @property
    def TCP_UDP_port(self) -> cdt.LongUnsigned:
        return self.get_attr(2)

    @property
    def IP_reference(self) -> cst.LogicalName:
        return self.get_attr(3)

    @property
    def MMS(self) -> cdt.LongUnsigned:
        return self.get_attr(4)

    @property
    def nb_of_sim_conn(self) -> cdt.Unsigned:
        return self.get_attr(5)

    @property
    def inactivity_time_out(self) -> cdt.LongUnsigned:
        return self.get_attr(6)
